from qualys_etl.etld_lib import etld_lib_config as etld_lib_config
from qualys_etl.etld_lib import etld_lib_functions as etld_lib_functions
from qualys_etl.etld_lib import etld_lib_extract_transform_load_distribute as etld_lib_extract_transform_load_distribute

global count_qid_loaded_to_json


def kb_json():
    global count_qid_loaded_to_json
    start_msg_kb_json()
    shelve_db = etld_lib_config.kb_shelve_file
    load_json_file = etld_lib_config.kb_json_file
    count_qid_loaded_to_json = etld_lib_extract_transform_load_distribute.load_json(load_json_file,shelve_db)
    end_msg_kb_json()


def start_msg_kb_json():
    etld_lib_functions.logger.info(f"start")


def end_msg_kb_json():
    global count_qid_loaded_to_json
    etld_lib_functions.logger.info(f"count qid loaded to json: {count_qid_loaded_to_json:,}")
    etld_lib_functions.log_file_info(etld_lib_config.kb_shelve_file, 'in')
    etld_lib_functions.log_dbm_info(etld_lib_config.kb_shelve_file)
    etld_lib_functions.log_file_info(etld_lib_config.kb_json_file)
    etld_lib_functions.logger.info(f"end")


def main():
    kb_json()


if __name__ == "__main__":
    etld_lib_functions.main(my_logger_prog_name="kb_load_json")
    etld_lib_config.main()
    main()



import shelve
import xmltodict
import time

from qualys_etl.etld_lib import etld_lib_config as etld_lib_config
from qualys_etl.etld_lib import etld_lib_functions as etld_lib_functions
global count_host_ids_added_to_shelve
global host_list_shelve_database
global host_list_xml_files_selected


def transform_host_list_item_to_shelve(_, item):
    global host_list_shelve_database
    global count_host_ids_added_to_shelve
    if len(_) > 2 and "HOST" != _[3][0]:
        return True
    ID = item['ID']
    count_host_ids_added_to_shelve = count_host_ids_added_to_shelve + 1
    # Retry logic to help with windows dbm.dumb slow writer.
    # TODO update for loop to else syntax.
    for i in range(5):
        try:
            host_list_shelve_database[ID] = item
            break
        except Exception as e:
            time.sleep(1)
            etld_lib_functions.logger.info(
                f"Retry #{i + 1:02d} writing ID {ID} to {str(etld_lib_config.host_list_shelve_file)} Exception {e}")
            if i > 4:
                etld_lib_functions.logger.error(f"Error Writing ID {ID} ")
                etld_lib_functions.logger.error(f"Error in File: {__file__} Line: {etld_lib_functions.lineno()}")
                etld_lib_functions.logger.error(f"Error writing to {str(etld_lib_config.kb_shelve_file)} ")
                etld_lib_functions.logger.error(f"Exception: {e}")
                exit(1)

    return True


def host_list_shelve():
    global count_host_ids_added_to_shelve
    global host_list_shelve_database
    global host_list_xml_files_selected
    count_host_ids_added_to_shelve = 0
    # xml_files, other = (not ec2, google or azure). (ec2,google,azure = include metadata for each)
    if etld_lib_config.host_list_payload_option == 'notags': # Process other with basic query options.
        host_list_xml_files_selected = {'host_list_other_xml_file': etld_lib_config.host_list_other_xml_file}
    else:
        host_list_xml_files_selected = {'host_list_other_xml_file': etld_lib_config.host_list_other_xml_file,
                                        'host_list_ec2_xml_file': etld_lib_config.host_list_ec2_xml_file,
                                        'host_list_gcp_xml_file': etld_lib_config.host_list_gcp_xml_file,
                                        'host_list_azure_xml_file': etld_lib_config.host_list_azure_xml_file}
    etld_lib_functions.log_dbm_info(etld_lib_config.host_list_shelve_file)
    try:
        for host_list_xml_file in host_list_xml_files_selected.values():
            with open(host_list_xml_file, 'r', encoding='utf-8') as xml_file:
                with shelve.open(str(etld_lib_config.host_list_shelve_file), flag='n') as host_list_shelve_database:
                    xmltodict.parse(xml_file.read(), item_depth=4, item_callback=transform_host_list_item_to_shelve)
    except Exception as e:
        etld_lib_functions.logger.error(f"XML File corruption detected: {host_list_xml_file}")
        etld_lib_functions.logger.error(f"Exception: {e}")
        exit(1)


def end_msg_host_list_shelve():
    global count_host_ids_added_to_shelve
    global host_list_xml_files_selected
    etld_lib_functions.logger.info(
        f"count host ids added to shelve: {count_host_ids_added_to_shelve:,} for {str(etld_lib_config.host_list_shelve_file)}")
    for host_list_xml_file in host_list_xml_files_selected.values():
        etld_lib_functions.log_file_info(host_list_xml_file, 'in')
    etld_lib_functions.log_dbm_info(etld_lib_config.host_list_shelve_file)
    etld_lib_functions.log_file_info(etld_lib_config.host_list_shelve_file)
    etld_lib_functions.logger.info(f"end")


def start_msg_host_list_shelve():
    etld_lib_functions.logger.info("start")


def main():
    start_msg_host_list_shelve()
    host_list_shelve()
    end_msg_host_list_shelve()


if __name__ == "__main__":
    etld_lib_functions.main(my_logger_prog_name='host_list_shelve')
    etld_lib_config.main()
    main()



import qualys_etl.etld_lib.etld_lib_config as etld_lib_config
import qualys_etl.etld_lib.etld_lib_spawn_etl as etld_lib_spawn_etl
import qualys_etl.etld_host_list_detection.etld_01_workflow_manager_host_list_detection as etl_host_list_detection


def main():
    etld_lib_config.set_path_qetl_user_home_dir()
    etld_lib_spawn_etl.log_dir = etld_lib_config.qetl_user_log_dir
    etld_lib_spawn_etl.log_file_path = etld_lib_config.host_list_detection_log_file
    etld_lib_spawn_etl.log_file_rotate_path = etld_lib_config.host_list_detection_log_rotate_file
    etld_lib_spawn_etl.lock_file = etld_lib_config.host_list_detection_lock_file
    etld_lib_spawn_etl.log_file_max_size = (1024 * 100000)          # 1024 * size = Max Meg Size
    etld_lib_spawn_etl.spawned_process_max_run_time = (60*120)       # (seconds * minutes), then terminate as there is an issue.
    etld_lib_spawn_etl.spawned_process_sleep_time = 5               # check every n seconds for spawned_process.is_alive()
    etld_lib_spawn_etl.spawned_process_count_to_status_update = 11  # Every n checks print status spawned_process.is_alive()

    etld_lib_spawn_etl.target_module_to_run = etl_host_list_detection.host_list_detection_etl_workflow
    etld_lib_spawn_etl.target_method_in_module_to_run = "host_list_detection_etl_workflow"
    etld_lib_spawn_etl.etl_main()


if __name__ == '__main__':
    main()



import sqlite3
import re
import csv
from qualys_etl.etld_lib import etld_lib_functions as etld_lib_functions
global count_rows_added_to_table


def create_table_statement(primary_key, table_name, csv_columns):
    table_statement = f"CREATE TABLE {table_name} (\n"
    table_statement = f"{table_statement}{primary_key} CHAR PRIMARY KEY NOT NULL,\n"
    for column in csv_columns:
        if primary_key != column:
            table_statement = f"{table_statement}{column} CHAR,\n"
    table_statement = re.sub(",\n$", "", table_statement)
    table_statement = table_statement + ");"
    return table_statement


def create_table(table_name, csv_columns, sqlite_file, key):
    try:
        conn = sqlite3.connect(sqlite_file)
        conn.execute(f"DROP TABLE IF EXISTS {table_name};")
        table_statement = create_table_statement(key, table_name, csv_columns)
        conn.execute(table_statement)
        conn.commit()
        conn.close()
    except Exception as e:
        etld_lib_functions.logger.error(f"Error in File: {__file__} Line: {etld_lib_functions.lineno()}")
        etld_lib_functions.logger.error(f"Error creating table: {table_name}, please retry after fixing error")
        etld_lib_functions.logger.error(f"Exception: {e}")
        exit(1)


def bulk_insert_csv_file(table_name, csv_file_name, csv_columns, sqlite_file):
    global count_rows_added_to_table
    try:
        values_var = ""
        for column in csv_columns:
            values_var = f"{values_var}?, "
        values_var = re.sub("\?, $", "?", values_var)
        csv_headers = ""
        csv_row_list = []
        bulk_insert_count = 100
        conn = sqlite3.connect(sqlite_file)
        cursor_obj = conn.cursor()
        with open(csv_file_name, "r", encoding='utf-8') as csv_file:
            csv_reader = csv.reader(csv_file, delimiter=',')
            count_rows_added_to_table = 0
            csv_headers = next(csv_reader)  # Skip Header
            insert_command = f"INSERT INTO {table_name} VALUES({values_var})"
            for row in csv_reader:
                csv_row_list.append(row)
                if len(csv_row_list) > bulk_insert_count:
                    cursor_obj.executemany(f"INSERT INTO {table_name} VALUES({values_var})", csv_row_list)
                    conn.commit()
                    csv_row_list = []
                count_rows_added_to_table = count_rows_added_to_table + 1
            if len(csv_row_list) > 0:
                cursor_obj.executemany(f"INSERT INTO {table_name} VALUES({values_var})", csv_row_list)
                conn.commit()
                csv_row_list = []
    except Exception as e:
        etld_lib_functions.logger.error(f"Error creating table: {csv_file_name}, please retry after fixing error")
        etld_lib_functions.logger.error(f"Exception: {e}")
        exit(1)


def create_table_statement_no_primary_key(table_name, csv_columns):
    table_statement = f"CREATE TABLE {table_name} (\n"
    table_statement = f"{table_statement}\n"
    for column in csv_columns:
        table_statement = f"{table_statement}{column} CHAR,\n"
    table_statement = re.sub(",\n$", "", table_statement)
    table_statement = table_statement + ");"
    return table_statement


def create_table_no_primary_key(table_name, csv_columns, sqlite_file):
    try:
        conn = sqlite3.connect(sqlite_file)
        conn.execute(f"DROP TABLE IF EXISTS {table_name};")
        table_statement = create_table_statement_no_primary_key(table_name, csv_columns)
        conn.execute(table_statement)
        conn.commit()
        conn.close()
    except Exception as e:
        etld_lib_functions.logger.error(f"Error creating table: {table_name}, please retry after fixing error")
        etld_lib_functions.logger.error(f"Exception: {e}")
        exit(1)


def get_q_knowledgebase_min_max_dates(sqlite_file):
    min_date = None
    max_date = None
    try:
        conn = sqlite3.connect(sqlite_file)
        cursor = conn.cursor()
        select_statement = 'select min(LAST_SERVICE_MODIFICATION_DATETIME) MIN_DATE, ' \
                           'max(LAST_SERVICE_MODIFICATION_DATETIME) MAX_DATE ' \
                           'from Q_KnowledgeBase;'
        cursor.execute(select_statement)
        records = cursor.fetchall()
        for row in records:
            min_date = row[0]
            max_date = row[1]
        conn.close()
    except Exception as e:
        min_date = 'Not Found'
        max_date = 'Not Found'

    return min_date, max_date


def create_table_q_knowledgebase_in_host_list_detection(sqlite_file):
    table_name = 'Q_KnowledgeBase_In_Host_List_Detection'
    kb_table_name = 'Q_KnowledgeBase'
    table_statement = f'CREATE TABLE {table_name} as ' \
                      f'select * from {kb_table_name} ' \
                      f'where QID in (select distinct(QID) from Q_Host_List_Detection);'
    try:
        conn = sqlite3.connect(sqlite_file)
        conn.execute(f"DROP TABLE IF EXISTS {table_name};")
        conn.execute(table_statement)
        conn.execute(f"DROP TABLE IF EXISTS {kb_table_name};")
        conn.commit()
        conn.close()
        conn = sqlite3.connect(sqlite_file)
        conn.isolation_level = None
        conn.execute("VACUUM")
        conn.close()
    except Exception as e:
        etld_lib_functions.logger.error(f"Error creating table: {table_name}, please retry after fixing error")
        etld_lib_functions.logger.error(f"Exception: {e}")
        exit(1)
